<?php

if (ini_get('safe_mode'))
	die("Script must be run in safe_mode=0\n");

$isWindows = stristr(PHP_OS, 'win') ? true : false;
if (!$isWindows)
	$plesk_dir = file_exists('/usr/local/psa') ? '/usr/local/psa' : '/opt/psa';
else
	$plesk_dir = chop(getenv('plesk_dir'), '/\\');

$version = "$plesk_dir/version";
if (!file_exists($version))
	die("The file \"$version\" has not been found.\n");

$version_s = chop(file_get_contents($version));
if (!preg_match('/^(\d?\d)\.(\d)\.(\d)/', $version_s, $v))
	die("Failed to get Plesk version from \"$version\"\n");

list($version, $major, $minor, $release) = $v;

$fixes = array();

$fixes[] = array(103, 103, 'SecretKey.php', 'admin/plib/api-common');
$fixes[] = array(81, 102, 'Agent.php', 'admin/plib/api-rpc');

if ($isWindows)
	$fixes[] = array(81, 103, 'visibility.php', 'admin/plib');

$fixes[] = array(81, 104, 'help.php', 'admin/htdocs');

$fixes[] = array(81, 104, 'common_func.php3', 'admin/plib');

$fixes[] = array(104, 104, 'AgentSubDomain.php', 'admin/plib/api-rpc');
$fixes[] = array(102, 103, 'AbstractAgentSubDomain.php', 'admin/plib/api-rpc');
$fixes[] = array(83, 101, 'AgentSubDomain.php', 'admin/plib/api-rpc');

if ($isWindows) {
	$fixes[] = array(81, 110, 'head.php3', 'admin/plib');
	$fixes[] = array(90, 110, 'elements.php3', 'admin/plib');
}

$majorminor = (int)"$major$minor";
foreach ($fixes as $fix) {
	if ($majorminor < $fix[0] || $majorminor > $fix[1])
		continue;

	$file = $fix[2];
	$fdir = $fix[3];

	$dest = "$plesk_dir/$fdir/$file";
	if (!file_exists($dest)) {
		print "The file \"$dest\" has not been found. The patch is not applicable.\n";
		continue;
	}

	if (file_exists($version)) {
		$src = "$version/$file";
	} else if (file_exists("$major.$minor.x")) {
		$src = "$major.$minor.x/$file";
	} else
		die("Unable to find patched '$file' for version $version\n");

	print "Copying \"$src\" to \"$dest\"\n";

	if (!copy($src, $dest))
		die("Failed to copy \"$src\" to \"$dest\"\n");
}

if (!$isWindows) {
	print "Restarting Plesk\n";
	system('/etc/init.d/psa restart', $ec);
	if ($ec != 0)
		die("Failed to restart Plesk. Please, run manually: /etc/init.d/psa restart\n");
}

print "The patch has been successfully applied.\n";
